#import "MainImageView.h"

@implementation MainImageView

#pragma mark Initializations
- (id) init
{
	self = [super init];
	if (self != nil) {
		hueAdjust = 0.0;
		discBlur = 0.0;
	}
	return self;
}

-(void)awakeFromNib
{
	[hueValue setFloatValue:0.0];
	[discBlurValue setFloatValue:0.0];
	
}

#pragma mark -
#pragma mark Drawing Functions
-(void)drawRect:(NSRect)rect
{
	
	NSLog(@"redrawing now...");
	[theImage drawInRect:[self bounds] fromRect:[self bounds] operation:NSCompositeSourceOver fraction:1.0];

}

- (IBAction)ApplyHue:(id)sender
{
	NSLog(@"applying hue adjust");
	CIImage *resultImage;
	CIImage *inputImage = [theImage CIImage];
	
	CIFilter *hueAdjustFilter = [CIFilter filterWithName:@"CIHueAdjust"];
	[hueAdjustFilter setDefaults];
	[hueAdjustFilter setValue:inputImage forKey:@"inputImage"];
	[hueAdjustFilter setValue:[NSNumber numberWithFloat:[hueValue floatValue]] forKey:@"inputAngle"];
	resultImage = [hueAdjustFilter valueForKey:@"outputImage"];
	[theImage release];
	theImage = [[NSImage imageFromCIImage:resultImage]retain];
	[self setNeedsDisplay:YES];
}

- (IBAction)applyDiscBlur:(id)sender
{
	NSLog(@"applying Disc Blur...");
	CIImage *inputImage = [theImage CIImage];
	CIImage *resultImage;
	CIFilter *discBlurFilter = [CIFilter filterWithName:@"CIDiscBlur"];
	[discBlurFilter setDefaults];
	[discBlurFilter setValue:inputImage forKey:@"inputImage"];
	[discBlurFilter setValue:[NSNumber numberWithFloat:[discBlurValue floatValue]] forKey:@"inputRadius"];
	
	resultImage = [discBlurFilter valueForKey:@"outputImage"];
	[theImage release];
	theImage = [[NSImage imageFromCIImage:resultImage]retain];
	[self setNeedsDisplay:YES];
}

#pragma mark -
#pragma mark Image Loading/Saving
-(void)loadImageFromPath:(NSString *)aPath
{
	[self resetAllValues];
	NSFileManager *fm = [NSFileManager defaultManager];

	if ([fm fileExistsAtPath:aPath]) {
		[theImage unlockFocus];
		[theImage release];
		theImage = [[NSImage alloc]initWithContentsOfFile:aPath];
		NSRect frame = [self frame];
		[self setFrame:NSMakeRect(frame.origin.x,frame.origin.y,[theImage size].width,[theImage size].height)];
		[theImage lockFocus];
		[self setBoundsSize:[theImage size]];
		[self setNeedsDisplay:YES];
	}
	else
		NSBeep();
}

- (IBAction)selectNewImage:(id)sender
{
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	if ([openPanel runModal] == NSOKButton) {
		NSLog([openPanel filename]);
		NSString *fileLocation = [openPanel filename];
		[self loadImageFromPath:fileLocation];
	}
}

- (IBAction)saveImage:(id)sender
{
	//NSData *data = [self dataWithPDFInsideRect:[self bounds]];
	NSData *data  = [theImage TIFFRepresentation];
	[data writeToFile:[[NSString stringWithString:@"~/Desktop/test.tiff"]stringByStandardizingPath] atomically:NO];
}

#pragma mark -
#pragma mark Adjusting Sliders and Values...
- (IBAction)hueAdjustSliderDidChange:(id)sender
{
	hueAdjust = [sender floatValue];
	[self setNeedsDisplay:YES];
}

- (IBAction)discBlurSliderDidChange:(id)sender
{
	discBlur = [sender floatValue];
	[self setNeedsDisplay:YES];
}

- (void)resetAllValues
{
	[zoomSlider setFloatValue:0.0];
	[discBlurSlider setFloatValue:0.0];
	[hueSlider setFloatValue:0.0];
}

- (IBAction)zoomSliderDidChange:(id)sender
{
	[self changeZoom:[zoomSlider floatValue]];
}

-(void)changeZoom:(float)floatValue
{
	NSRect frame = [self frame];
	NSRect bounds = [self bounds];
	frame.size.width = bounds.size.width * (floatValue/100);
    frame.size.height = bounds.size.height *(floatValue/100);
	[self setFrameSize: frame.size];    // Change the view's size.
    [self setBoundsSize: bounds.size];  // Restore the view's bounds, which causes the view to be scaled.
	[self setNeedsDisplay:YES];	
}

#pragma mark -
#pragma mark Notifications
-(void)windowWillClose:(NSNotification *)aNotification
{
	[NSApp terminate:self];
}


#pragma mark -
#pragma mark deallocations...
- (void) dealloc
{
	[theImage release];
	[super dealloc];
}


@end
